import os
import platform
import shutil

import pytest

import py7zr
import py7zr.helpers

testdata_path = os.path.join(os.path.dirname(__file__), "data")
targets = [
    ("zstd", [{"id": py7zr.FILTER_ZSTD, "level": 3}]),
    ("bzip2", [{"id": py7zr.FILTER_BZIP2}]),
    ("lzma+bcj", [{"id": py7zr.FILTER_X86}, {"id": py7zr.FILTER_LZMA, "preset": 7}]),
    ("lzma2+bcj", [{"id": py7zr.FILTER_X86}, {"id": py7zr.FILTER_LZMA2, "preset": 7}]),
    (
        "bzip2+aes",
        [{"id": py7zr.FILTER_BZIP2}, {"id": py7zr.FILTER_CRYPTO_AES256_SHA256}],
    ),
    (
        "lzma2+aes",
        [
            {"id": py7zr.FILTER_LZMA2, "preset": 7},
            {"id": py7zr.FILTER_CRYPTO_AES256_SHA256},
        ],
    ),
]


@pytest.mark.benchmark(group="compress")
@pytest.mark.parametrize("name, filters", targets)
def test_benchmark_filters_compress(tmp_path, benchmark, name, filters):
    def compressor(filters, password):
        with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", filters=filters, password=password) as szf:
            szf.writeall(tmp_path.joinpath("src"), "src")

    def setup():
        if tmp_path.joinpath("target.7z").exists():
            tmp_path.joinpath("target.7z").unlink()

    with py7zr.SevenZipFile(os.path.join(testdata_path, "mblock_1.7z"), "r") as szf:
        szf.extractall(path=tmp_path.joinpath("src"))
    with py7zr.SevenZipFile(os.path.join(testdata_path, "mblock_1.7z"), "r") as szf:
        archive_info = szf.archiveinfo()
        source_size = archive_info.uncompressed
    if name.endswith("aes"):
        password = "secret"
    else:
        password = None
    benchmark.extra_info["data_size"] = source_size
    benchmark.pedantic(compressor, setup=setup, args=[filters, password], iterations=1, rounds=3)
    benchmark.extra_info["ratio"] = str(tmp_path.joinpath("target.7z").stat().st_size / source_size)


@pytest.mark.benchmark(group="decompress")
@pytest.mark.parametrize("name, filters", targets)
def test_benchmark_filters_decompress(tmp_path, benchmark, name, filters):
    def decompressor(secret):
        with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "r", password=secret) as szf:
            szf.extractall(tmp_path.joinpath("tgt"))

    def setup():
        shutil.rmtree(tmp_path.joinpath("tgt"), ignore_errors=True)

    with py7zr.SevenZipFile(os.path.join(testdata_path, "mblock_1.7z"), "r") as szf:
        szf.extractall(path=tmp_path.joinpath("src"))
    with py7zr.SevenZipFile(os.path.join(testdata_path, "mblock_1.7z"), "r") as szf:
        archive_info = szf.archiveinfo()
        source_size = archive_info.uncompressed

    if name.endswith("aes"):
        password = "secret"
    else:
        password = None
    with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", filters=filters, password=password) as szf:
        szf.writeall(tmp_path.joinpath("src"), "src")
    benchmark.extra_info["data_size"] = source_size
    benchmark.extra_info["ratio"] = str(tmp_path.joinpath("target.7z").stat().st_size / source_size)
    benchmark.pedantic(decompressor, setup=setup, args=[password], iterations=1, rounds=3)


textfilters = [
    ("ppmd(text)", [{"id": py7zr.FILTER_PPMD, "order": 8, "mem": "4m"}]),
    ("deflate(text)", [{"id": py7zr.FILTER_DEFLATE}]),
    ("zstd(text)", [{"id": py7zr.FILTER_ZSTD, "level": 3}]),
    ("brotli(text)", [{"id": py7zr.FILTER_BROTLI, "level": 11}]),
]


@pytest.mark.benchmark(group="compress")
@pytest.mark.parametrize("name, filters", textfilters)
def test_benchmark_text_compress(tmp_path, benchmark, name, filters):
    def compressor(filters):
        with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", filters=filters) as szf:
            szf.writeall(tmp_path.joinpath("src"), "src")

    def setup():
        if tmp_path.joinpath("target.7z").exists():
            tmp_path.joinpath("target.7z").unlink()

    with py7zr.SevenZipFile(os.path.join(testdata_path, "bzip2_2.7z"), "r") as szf:
        szf.extractall(path=tmp_path.joinpath("src"))
    with py7zr.SevenZipFile(os.path.join(testdata_path, "bzip2_2.7z"), "r") as szf:
        archive_info = szf.archiveinfo()
        source_size = archive_info.uncompressed
    benchmark.extra_info["data_size"] = source_size
    benchmark.pedantic(compressor, setup=setup, args=[filters], iterations=1, rounds=3)
    benchmark.extra_info["ratio"] = str(tmp_path.joinpath("target.7z").stat().st_size / source_size)


@pytest.mark.benchmark(group="decompress")
@pytest.mark.parametrize("name, filters", textfilters)
def test_benchmark_text_decompress(tmp_path, benchmark, name, filters):
    def decompressor():
        with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "r") as szf:
            szf.extractall(tmp_path.joinpath("tgt"))

    def setup():
        shutil.rmtree(tmp_path.joinpath("tgt"), ignore_errors=True)

    with py7zr.SevenZipFile(os.path.join(testdata_path, "bzip2_2.7z"), "r") as szf:
        szf.extractall(path=tmp_path.joinpath("src"))
    with py7zr.SevenZipFile(os.path.join(testdata_path, "bzip2_2.7z"), "r") as szf:
        archive_info = szf.archiveinfo()
        source_size = archive_info.uncompressed
    password = None
    with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", filters=filters, password=password) as szf:
        szf.writeall(tmp_path.joinpath("src"), "src")
    benchmark.extra_info["data_size"] = source_size
    benchmark.extra_info["ratio"] = str(tmp_path.joinpath("target.7z").stat().st_size / source_size)
    benchmark.pedantic(decompressor, setup=setup, args=[], iterations=1, rounds=3)


@pytest.mark.benchmark(group="calculate_key")
@pytest.mark.skip(reason="Don't test in ordinary development")
def test_benchmark_calculate_key1(benchmark):
    password = "secret".encode("utf-16LE")
    cycles = 19
    salt = b""
    expected = b"e\x11\xf1Pz<*\x98*\xe6\xde\xf4\xf6X\x18\xedl\xf2Be\x1a\xca\x19\xd1\\\xeb\xc6\xa6z\xe2\x89\x1d"
    key = benchmark(py7zr.helpers._calculate_key1, password, cycles, salt, "sha256")
    assert key == expected


@pytest.mark.benchmark(group="calculate_key")
@pytest.mark.skip(reason="Don't test in ordinary development")
@pytest.mark.skipif(platform.python_implementation() == "PyPy", reason="Will crash on PyPy")
def test_benchmark_calculate_key2(benchmark):
    password = "secret".encode("utf-16LE")
    cycles = 19
    salt = b""
    expected = b"e\x11\xf1Pz<*\x98*\xe6\xde\xf4\xf6X\x18\xedl\xf2Be\x1a\xca\x19\xd1\\\xeb\xc6\xa6z\xe2\x89\x1d"
    key = benchmark(py7zr.helpers._calculate_key2, password, cycles, salt, "sha256")
    assert key == expected


@pytest.mark.benchmark(group="calculate_key")
@pytest.mark.skip(reason="Don't test in ordinary development")
def test_benchmark_calculate_key3(benchmark):
    password = "secret".encode("utf-16LE")
    cycles = 19
    salt = b""
    expected = b"e\x11\xf1Pz<*\x98*\xe6\xde\xf4\xf6X\x18\xedl\xf2Be\x1a\xca\x19\xd1\\\xeb\xc6\xa6z\xe2\x89\x1d"
    key = benchmark(py7zr.helpers._calculate_key3, password, cycles, salt, "sha256")
    assert key == expected



# Configuration for pytest to automatically collect types.
# Thanks to Guilherme Salgado.
import os

import cpuinfo
import pytest
from pyannotate_runtime import collect_types


def pytest_collection_finish(session):
    """Handle the pytest collection finish hook: configure pyannotate.
    Explicitly delay importing `collect_types` until all tests have
    been collected.  This gives gevent a chance to monkey patch the
    world before importing pyannotate.
    """
    collect_types.init_types_collection()


@pytest.fixture(autouse=True)
def collect_types_fixture():
    collect_types.start()
    yield
    collect_types.stop()


def pytest_sessionfinish(session, exitstatus):
    os.makedirs("build/", exist_ok=True)
    collect_types.dump_stats("build/type_info.json")


def pytest_benchmark_update_json(config, benchmarks, output_json):
    """Calculate compression/decompression speed and add as extra_info"""
    for benchmark in output_json["benchmarks"]:
        if "data_size" in benchmark["extra_info"]:
            rate = benchmark["extra_info"].get("data_size", 0.0) / benchmark["stats"]["mean"]
            benchmark["extra_info"]["rate"] = rate


def pytest_benchmark_update_machine_info(config, machine_info):
    cpu_info = cpuinfo.get_cpu_info()
    brand = cpu_info.get("brand_raw", None)
    if brand is None:
        brand = "{} core(s) {} CPU ".format(cpu_info.get("count", "unknown"), cpu_info.get("arch", "unknown"))
    machine_info["cpu"]["brand"] = brand
    machine_info["cpu"]["hz_actual_friendly"] = cpu_info.get("hz_actual_friendly", "unknown")


def pytest_addoption(parser):
    parser.addoption("--run-slow", action="store_true", default=False, help="run slow tests")


def pytest_collection_modifyitems(config, items):
    if config.getoption("--run-slow"):
        # --run-slow given in cli: do not skip slow tests
        return
    skip_slow = pytest.mark.skip(reason="need --run-slow option to run")
    for item in items:
        if "slow" in item.keywords:
            item.add_marker(skip_slow)



import argparse
import os
import pathlib

import github
import bench_result as bench


class CommentBenchmark:

    def __init__(self):
        parser = argparse.ArgumentParser(prog='bench_comment_issue')
        parser.add_argument('jsonfile', type=pathlib.Path, help='pytest-benchmark saved result')
        parser.add_argument('repository', help='Repository to post comment')
        parser.add_argument('issue_number', type=int, help='Issue number to post comment')
        parser.add_argument('run_id', help='Run ID of github actions')
        args = parser.parse_args()
        self.jsonfile = args.jsonfile
        self.repository = args.repository
        self.issue_number = args.issue_number
        self.run_id = args.run_id

    def post_comment(self):
        body = bench.generate_comment(self.jsonfile, type='github')
        body += '\n   Posted from [the action](https://github.com/{}/actions/runs/{})\n'.format(self.repository, self.run_id)
        token = os.getenv("GITHUB_TOKEN")
        g = github.Github(token)
        repo = g.get_repo(self.repository)
        issue = repo.get_issue(number=self.issue_number)
        issue.create_comment(body)


if __name__ == "__main__":
    CommentBenchmark().post_comment()



import asyncio
import binascii
import ctypes
import hashlib
import os
import pathlib
import shutil
import subprocess
import sys
from datetime import datetime

import pytest

import py7zr
from py7zr import unpack_7zarchive
from py7zr.exceptions import CrcError, UnsupportedCompressionMethodError
from py7zr.helpers import UTC

from . import aio7zr, decode_all

testdata_path = pathlib.Path(os.path.dirname(__file__)).joinpath("data")
os.umask(0o022)


def check_archive(archive, tmp_path, return_dict: bool):
    assert sorted(archive.getnames()) == ["test", "test/test2.txt", "test1.txt"]
    expected = []
    expected.append({"filename": "test"})
    expected.append(
        {
            "lastwritetime": 12786932616,
            "as_datetime": datetime(2006, 3, 15, 21, 43, 36, 0, UTC()),
            "filename": "test/test2.txt",
        }
    )
    expected.append(
        {
            "lastwritetime": 12786932628,
            "as_datetime": datetime(2006, 3, 15, 21, 43, 48, 0, UTC()),
            "filename": "test1.txt",
        }
    )
    for i, cf in enumerate(archive.files):
        assert cf.filename == expected[i]["filename"]
        if not cf.is_directory:
            assert cf.lastwritetime // 10000000 == expected[i]["lastwritetime"]
            assert cf.lastwritetime.as_datetime().replace(microsecond=0) == expected[i]["as_datetime"]
    if not return_dict:
        archive.extractall(path=tmp_path)
        assert tmp_path.joinpath("test/test2.txt").open("rb").read() == bytes("This file is located in a folder.", "ascii")
        assert tmp_path.joinpath("test1.txt").open("rb").read() == bytes("This file is located in the root.", "ascii")
    else:
        _dict = archive.readall()
        actual = _dict["test/test2.txt"].read()
        assert actual == bytes("This file is located in a folder.", "ascii")
        actual = _dict["test1.txt"].read()
        assert actual == bytes("This file is located in the root.", "ascii")
    archive.close()


@pytest.mark.files
def test_solid(tmp_path):
    f = "solid.7z"
    archive = py7zr.SevenZipFile(testdata_path.joinpath(f).open(mode="rb"))
    check_archive(archive, tmp_path, False)


@pytest.mark.files
def test_solid_mem(tmp_path):
    f = "solid.7z"
    archive = py7zr.SevenZipFile(testdata_path.joinpath(f).open(mode="rb"))
    check_archive(archive, tmp_path, True)


@pytest.mark.files
def test_empty():
    # decompress empty archive
    archive = py7zr.SevenZipFile(testdata_path.joinpath("empty.7z").open(mode="rb"))
    assert archive.getnames() == []


@pytest.mark.files
def test_github_14(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("github_14.7z").open(mode="rb"))
    assert archive.getnames() == ["github_14"]
    archive.extractall(path=tmp_path)
    with tmp_path.joinpath("github_14").open("rb") as f:
        assert f.read() == bytes("Hello GitHub issue #14.\n", "ascii")


@pytest.mark.files
def test_github_14_mem(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("github_14.7z").open(mode="rb"))
    _dict = archive.readall()
    actual = _dict["github_14"].read()
    assert actual == bytes("Hello GitHub issue #14.\n", "ascii")


@pytest.mark.files
def _test_umlaut_archive(filename: str, target: pathlib.Path, return_dict: bool):
    archive = py7zr.SevenZipFile(testdata_path.joinpath(filename).open(mode="rb"))
    if not return_dict:
        assert sorted(archive.getnames()) == ["t\xe4st.txt"]
        archive.extractall(path=target)
        actual = target.joinpath("t\xe4st.txt").open().read()
        assert actual == "This file contains a german umlaut in the filename."
    else:
        _dict = archive.readall()
        actual = _dict["t\xe4st.txt"].read()
        assert actual == b"This file contains a german umlaut in the filename."
    archive.close()


@pytest.mark.files
def test_non_solid_umlaut(tmp_path):
    # test loading of a non-solid archive containing files with umlauts
    _test_umlaut_archive("umlaut-non_solid.7z", tmp_path, False)


@pytest.mark.files
def test_non_solid_umlaut_mem(tmp_path):
    # test loading of a non-solid archive containing files with umlauts
    _test_umlaut_archive("umlaut-non_solid.7z", tmp_path, True)


@pytest.mark.files
def test_solid_umlaut(tmp_path):
    # test loading of a solid archive containing files with umlauts
    _test_umlaut_archive("umlaut-solid.7z", tmp_path, False)


@pytest.mark.files
def test_solid_umlaut_mem(tmp_path):
    # test loading of a solid archive containing files with umlauts
    _test_umlaut_archive("umlaut-solid.7z", tmp_path, True)


@pytest.mark.files
def test_bugzilla_4(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("bugzilla_4.7z").open(mode="rb"))
    expected = [
        {
            "filename": "History.txt",
            "mtime": 1133704668,
            "mode": 33188,
            "digest": "46b08f0af612371860ab39e3b47666c3bd6fb742c5e8775159310e19ebedae7e",
        },
        {
            "filename": "License.txt",
            "mtime": 1105356710,
            "mode": 33188,
            "digest": "4f49a4448499449f2864777c895f011fb989836a37990ae1ca532126ca75d25e",
        },
        {
            "filename": "copying.txt",
            "mtime": 999116366,
            "mode": 33188,
            "digest": "2c3c3ef532828bcd42bb3127349625a25291ff5ae7e6f8d42e0fe9b5be836a99",
        },
        {
            "filename": "readme.txt",
            "mtime": 1133704646,
            "mode": 33188,
            "digest": "84f2693d9746e919883cf169fc83467be6566d7501b5044693a2480ab36a4899",
        },
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.files
@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_extract_symlink(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("symlink.7z").open(mode="rb"))
    assert sorted(archive.getnames()) == [
        "lib",
        "lib/libabc.so",
        "lib/libabc.so.1",
        "lib/libabc.so.1.2",
        "lib/libabc.so.1.2.3",
        "lib64",
    ]
    archive.extractall(path=tmp_path)


@pytest.mark.files
def test_extract_symlink_mem():
    with py7zr.SevenZipFile(testdata_path.joinpath("symlink.7z").open(mode="rb")) as archive:
        _dict = archive.readall()


@pytest.mark.files
def test_lzma2bcj(tmp_path):
    """Test extract archive compressed with LZMA2 and BCJ methods."""
    archive = py7zr.SevenZipFile(testdata_path.joinpath("lzma2bcj.7z").open(mode="rb"))
    assert archive.getnames() == [
        "mingw64",
        "mingw64/bin",
        "mingw64/include",
        "mingw64/lib",
        "mingw64/share",
        "mingw64/share/doc",
        "mingw64/share/doc/szip",
        "mingw64/include/SZconfig.h",
        "mingw64/include/ricehdf.h",
        "mingw64/include/szip_adpt.h",
        "mingw64/include/szlib.h",
        "mingw64/lib/libszip.a",
        "mingw64/lib/libszip.dll.a",
        "mingw64/share/doc/szip/COPYING",
        "mingw64/share/doc/szip/HISTORY.txt",
        "mingw64/share/doc/szip/INSTALL",
        "mingw64/share/doc/szip/README",
        "mingw64/share/doc/szip/RELEASE.txt",
        "mingw64/bin/libszip-0.dll",
    ]
    archive.extractall(path=tmp_path)
    m = hashlib.sha256()
    m.update(tmp_path.joinpath("mingw64/bin/libszip-0.dll").open("rb").read())
    assert m.digest() == binascii.unhexlify("13926e3f080c9ca557165864ce5722acc4f832bb52a92d8d86c7f6e583708c4d")
    archive.close()


@pytest.mark.files
def test_lzma2bcj_mem():
    """Test extract archive compressed with LZMA2 and BCJ methods."""
    archive = py7zr.SevenZipFile(testdata_path.joinpath("lzma2bcj.7z").open(mode="rb"))
    assert archive.getnames() == [
        "mingw64",
        "mingw64/bin",
        "mingw64/include",
        "mingw64/lib",
        "mingw64/share",
        "mingw64/share/doc",
        "mingw64/share/doc/szip",
        "mingw64/include/SZconfig.h",
        "mingw64/include/ricehdf.h",
        "mingw64/include/szip_adpt.h",
        "mingw64/include/szlib.h",
        "mingw64/lib/libszip.a",
        "mingw64/lib/libszip.dll.a",
        "mingw64/share/doc/szip/COPYING",
        "mingw64/share/doc/szip/HISTORY.txt",
        "mingw64/share/doc/szip/INSTALL",
        "mingw64/share/doc/szip/README",
        "mingw64/share/doc/szip/RELEASE.txt",
        "mingw64/bin/libszip-0.dll",
    ]
    _dict = archive.readall()
    m = hashlib.sha256()
    m.update(_dict["mingw64/bin/libszip-0.dll"].read())
    assert m.digest() == binascii.unhexlify("13926e3f080c9ca557165864ce5722acc4f832bb52a92d8d86c7f6e583708c4d")
    archive.close()


@pytest.mark.files
def test_lzma2bcj2(tmp_path):
    """Test extract archive compressed with LZMA2 and BCJ2 methods."""
    with pytest.raises(UnsupportedCompressionMethodError):
        with testdata_path.joinpath("lzma2bcj2.7z").open(mode="rb") as target:
            archive = py7zr.SevenZipFile(target)
            archive.extractall(path=tmp_path)
            archive.close()


@pytest.mark.files
def test_lzma2bcj2_2(tmp_path):
    """Test extract archive compressed with LZMA2 and BCJ2 in multiple chunks."""
    with pytest.raises(UnsupportedCompressionMethodError):
        with testdata_path.joinpath("lzma2bcj2_2.7z").open(mode="rb") as target:
            archive = py7zr.SevenZipFile(target)
            archive.extractall(path=tmp_path)
            archive.close()


@pytest.mark.files
def test_extract_lzma_1(tmp_path):
    with testdata_path.joinpath("lzma_1.7z").open(mode="rb") as target:
        with py7zr.SevenZipFile(target) as ar:
            ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_lzma2_1(tmp_path):
    with testdata_path.joinpath("lzma2_1.7z").open(mode="rb") as target:
        with py7zr.SevenZipFile(target) as ar:
            _dict = ar.readall()


@pytest.mark.files
def test_zerosize(tmp_path):
    with testdata_path.joinpath("zerosize.7z").open(mode="rb") as target:
        archive = py7zr.SevenZipFile(target)
        archive.extractall(path=tmp_path)
        archive.close()


@pytest.mark.files
def test_zerosize_mem():
    with testdata_path.joinpath("zerosize.7z").open(mode="rb") as target:
        archive = py7zr.SevenZipFile(target)
        _dict = archive.readall()
        archive.close()


@pytest.mark.api
def test_register_unpack_archive(tmp_path):
    shutil.register_unpack_format("7zip", [".7z"], unpack_7zarchive)
    shutil.unpack_archive(str(testdata_path.joinpath("test_1.7z")), str(tmp_path))
    target = tmp_path.joinpath("setup.cfg")
    expected_mode = 33188
    expected_mtime = 1552522033
    if os.name == "posix":
        assert target.stat().st_mode == expected_mode
    assert target.stat().st_mtime == expected_mtime
    m = hashlib.sha256()
    m.update(target.open("rb").read())
    assert m.digest() == binascii.unhexlify("ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240")
    m = hashlib.sha256()
    m.update(tmp_path.joinpath("setup.py").open("rb").read())
    assert m.digest() == binascii.unhexlify("b916eed2a4ee4e48c51a2b51d07d450de0be4dbb83d20e67f6fd166ff7921e49")
    m = hashlib.sha256()
    m.update(tmp_path.joinpath("scripts/py7zr").open("rb").read())
    assert m.digest() == binascii.unhexlify("b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd")


@pytest.mark.files
def test_skip():
    archive = py7zr.SevenZipFile(testdata_path.joinpath("test_1.7z").open(mode="rb"))
    for i, cf in enumerate(archive.files):
        assert cf is not None
        archive.worker.register_filelike(cf.id, None)
    archive.worker.extract(archive.fp, None, parallel=True)
    archive.close()


@pytest.mark.files
def test_github_14_multi(tmp_path):
    """multiple unnamed objects."""
    archive = py7zr.SevenZipFile(str(testdata_path.joinpath("github_14_multi.7z")), "r")
    assert archive.getnames() == ["github_14_multi", "github_14_multi"]
    archive.extractall(path=tmp_path)
    with tmp_path.joinpath("github_14_multi").open("rb") as f:
        assert f.read() == bytes("Hello GitHub issue #14 1/2.\n", "ascii")
    with tmp_path.joinpath("github_14_multi_0").open("rb") as f:
        assert f.read() == bytes("Hello GitHub issue #14 2/2.\n", "ascii")
    archive.close()


@pytest.mark.files
def test_github_14_multi_mem():
    """multiple unnamed objects."""
    archive = py7zr.SevenZipFile(str(testdata_path.joinpath("github_14_multi.7z")), "r")
    assert archive.getnames() == ["github_14_multi", "github_14_multi"]
    _dict = archive.readall()
    actual_1 = _dict["github_14_multi"].read()
    assert actual_1 == bytes("Hello GitHub issue #14 1/2.\n", "ascii")
    actual_2 = _dict["github_14_multi_0"].read()
    assert actual_2 == bytes("Hello GitHub issue #14 2/2.\n", "ascii")
    archive.close()


@pytest.mark.files
def test_multiblock(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("mblock_1.7z").open(mode="rb"))
    archive.extractall(path=tmp_path)
    m = hashlib.sha256()
    m.update(tmp_path.joinpath("bin/7zdec.exe").open("rb").read())
    assert m.digest() == binascii.unhexlify("e14d8201c5c0d1049e717a63898a3b1c7ce4054a24871daebaa717da64dcaff5")
    archive.close()


@pytest.mark.files
def test_multiblock_mem():
    archive = py7zr.SevenZipFile(testdata_path.joinpath("mblock_1.7z").open(mode="rb"))
    _dict = archive.readall()
    m = hashlib.sha256()
    m.update(_dict["bin/7zdec.exe"].read())
    assert m.digest() == binascii.unhexlify("e14d8201c5c0d1049e717a63898a3b1c7ce4054a24871daebaa717da64dcaff5")
    archive.close()


@pytest.mark.files
@pytest.mark.skipif(sys.platform.startswith("win"), reason="Cannot unlink opened file on Windows")
def test_multiblock_unlink(tmp_path):
    """When passing opened file object, even after unlink it should work."""
    shutil.copy(str(testdata_path.joinpath("mblock_1.7z")), str(tmp_path))
    src = tmp_path.joinpath("mblock_1.7z")
    archive = py7zr.SevenZipFile(open(str(src), "rb"))
    os.unlink(str(src))
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_copy(tmp_path):
    """test loading of copy compressed files.(help wanted)"""
    check_archive(
        py7zr.SevenZipFile(testdata_path.joinpath("copy.7z").open(mode="rb")),
        tmp_path,
        False,
    )


@pytest.mark.files
def test_copy_2(tmp_path):
    """test loading of copy compressed files part2."""
    with py7zr.SevenZipFile(testdata_path.joinpath("copy_2.7z").open(mode="rb")) as ar:
        ar.extractall(path=tmp_path)


@pytest.mark.files
def test_close_unlink(tmp_path):
    shutil.copyfile(str(testdata_path.joinpath("test_1.7z")), str(tmp_path.joinpath("test_1.7z")))
    archive = py7zr.SevenZipFile(str(tmp_path.joinpath("test_1.7z")))
    archive.extractall(path=str(tmp_path))
    archive.close()
    tmp_path.joinpath("test_1.7z").unlink()


@pytest.mark.files
@pytest.mark.asyncio
@pytest.mark.skipif(hasattr(sys, "pypy_version_info"), reason="Not working with pypy3")
def test_asyncio_executor(tmp_path):
    shutil.copyfile(os.path.join(testdata_path, "test_1.7z"), str(tmp_path.joinpath("test_1.7z")))
    loop = asyncio.get_event_loop()
    task = asyncio.ensure_future(aio7zr(tmp_path.joinpath("test_1.7z"), path=tmp_path))
    loop.run_until_complete(task)
    loop.run_until_complete(asyncio.sleep(3))
    os.unlink(str(tmp_path.joinpath("test_1.7z")))


@pytest.mark.files
def test_no_main_streams(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("test_folder.7z").open(mode="rb"))
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_no_main_streams_mem():
    archive = py7zr.SevenZipFile(testdata_path.joinpath("test_folder.7z").open(mode="rb"))
    _dict = archive.readall()
    archive.close()


@pytest.mark.files
@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_extract_symlink_with_relative_target_path(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("symlink.7z").open(mode="rb"))
    os.chdir(str(tmp_path))
    os.makedirs(str(tmp_path.joinpath("target")))  # py35 need str() against pathlib.Path
    archive.extractall(path="target")
    assert os.readlink(str(tmp_path.joinpath("target/lib/libabc.so.1.2"))) == "libabc.so.1.2.3"
    archive.close()


@pytest.mark.files
@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_extract_emptystream_mix(tmp_path):
    archive = py7zr.SevenZipFile(str(testdata_path.joinpath("test_6.7z")), "r")
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_extract_longpath_file(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("longpath.7z").open("rb")) as archive:
        archive.extractall(path=tmp_path)


@pytest.mark.files
@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_extract_symlink_overwrite(tmp_path):
    os.chdir(str(tmp_path))
    os.makedirs(str(tmp_path.joinpath("target")))  # py35 need str() against pathlib.Path
    with py7zr.SevenZipFile(testdata_path.joinpath("symlink.7z").open(mode="rb")) as archive:
        archive.extractall(path="target")
    with py7zr.SevenZipFile(testdata_path.joinpath("symlink.7z").open(mode="rb")) as archive:
        archive.extractall(path="target")
    assert os.readlink(str(tmp_path.joinpath("target/lib/libabc.so.1.2"))) == "libabc.so.1.2.3"


@pytest.mark.files
def test_py7zr_extract_corrupted(tmp_path):
    with pytest.raises(CrcError):
        archive = py7zr.SevenZipFile(str(testdata_path.joinpath("crc_corrupted.7z")), "r")
        archive.extract(path=tmp_path)
        archive.close()


@pytest.mark.files
def test_extract_lzma2delta(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma2delta_1.7z").open("rb")) as archive:
        archive.extractall(path=tmp_path)


@pytest.mark.skipif(not shutil.which("7z"), reason="no 7z command installed")
def test_decompress_small_files(tmp_path):
    tmp_path.joinpath("t").mkdir()
    with tmp_path.joinpath("t/a").open("w") as f:
        f.write("1")
    with tmp_path.joinpath("t/b").open("w") as f:
        f.write("2")
    result = subprocess.run(
        ["7z", "a", (tmp_path / "target.7z").as_posix(), (tmp_path / "t").as_posix()],
        stdout=subprocess.PIPE,
    )
    if result.returncode != 0:
        print(result.stdout)
        pytest.fail("7z command report error")
    #
    with py7zr.SevenZipFile(tmp_path / "target.7z", "r") as arc:
        arc.testzip()


@pytest.mark.files
def test_extract_lzma_bcj_x86(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_x86.7z").open(mode="rb")) as ar:
        _dict = ar.readall()


@pytest.mark.files
def test_extract_lzma_bcj_arm(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_arm.7z").open(mode="rb")) as ar:
        ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_lzma_bcj_armt(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_armt.7z").open(mode="rb")) as ar:
        ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_lzma_bcj_ppc(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_ppc.7z").open(mode="rb")) as ar:
        ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_lzma_bcj_sparc(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_sparc.7z").open(mode="rb")) as ar:
        ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_lzma_bcj_2(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("lzma_bcj_2.7z").open(mode="rb")) as ar:
        ar.extractall(tmp_path)


@pytest.mark.files
def test_extract_hidden_linux_folder(tmp_path):
    hidden_folder_name = ".hidden_folder"
    with py7zr.SevenZipFile(testdata_path.joinpath("hidden_linux_folder.7z").open(mode="rb")) as archive:
        assert sorted(archive.getnames()) == [
            hidden_folder_name,
        ]
        archive.extractall(path=tmp_path)
        assert tmp_path.joinpath(hidden_folder_name).exists()


@pytest.mark.files
def test_extract_hidden_linux_file(tmp_path):
    hidden_file_name = ".hidden_file.txt"
    with py7zr.SevenZipFile(testdata_path.joinpath("hidden_linux_file.7z").open(mode="rb")) as archive:
        assert sorted(archive.getnames()) == [
            hidden_file_name,
        ]
        archive.extractall(path=tmp_path)
        assert tmp_path.joinpath(hidden_file_name).exists()



import ctypes
import os
import pathlib
import shutil
import subprocess
import sys

import pytest

import py7zr.archiveinfo
import py7zr.compressor
import py7zr.helpers
import py7zr.properties
from py7zr import PasswordRequired
from py7zr.properties import CompressionMethod

testdata_path = pathlib.Path(os.path.dirname(__file__)).joinpath("data")
os.umask(0o022)


@pytest.mark.files
def test_extract_encrypted_1(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("encrypted_1.7z").open(mode="rb"), password="secret")
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_extract_encrypted_1_mem():
    archive = py7zr.SevenZipFile(testdata_path.joinpath("encrypted_1.7z").open(mode="rb"), password="secret")
    _dict = archive.readall()
    archive.close()


@pytest.mark.files
def test_extract_encrypted_no_password(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("encrypted_1.7z").open(mode="rb"), password=None) as archive:
        with pytest.raises(PasswordRequired):
            archive.extractall(path=tmp_path)


@pytest.mark.files
def test_extract_encrypted_needs_password(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("encrypted_1.7z").open(mode="rb"), password=None) as archive:
        assert archive.needs_password()


@pytest.mark.files
def test_extract_header_encrypted_no_password(tmp_path):
    with pytest.raises(PasswordRequired):
        with py7zr.SevenZipFile(testdata_path.joinpath("encrypted_3.7z").open(mode="rb"), password=None) as archive:
            archive.extractall(path=tmp_path)


@pytest.mark.files
def test_extract_header_encrypted_no_password_2(tmp_path):
    with pytest.raises(PasswordRequired):
        with py7zr.SevenZipFile(testdata_path.joinpath("encrypted_4.7z").open(mode="rb"), password=None) as archive:
            archive.extractall(path=tmp_path)


@pytest.mark.files
@pytest.mark.timeout(45)
@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_extract_encrypted_2(tmp_path):
    with testdata_path.joinpath("encrypted_2.7z").open(mode="rb") as target:
        archive = py7zr.SevenZipFile(target, password="secret")
        assert archive.header.main_streams.unpackinfo.folders[0].coders[0]["method"] == CompressionMethod.CRYPT_AES256_SHA256
        assert archive.header.main_streams.unpackinfo.folders[0].coders[1]["method"] == CompressionMethod.LZMA2
        assert archive.header.main_streams.unpackinfo.folders[1].coders[0]["method"] == CompressionMethod.CRYPT_AES256_SHA256
        assert archive.header.main_streams.unpackinfo.folders[1].coders[1]["method"] == CompressionMethod.LZMA2
        assert archive.header.main_streams.unpackinfo.folders[1].coders[2]["method"] == CompressionMethod.P7Z_BCJ
        archive.extractall(path=tmp_path)
        archive.close()


@pytest.mark.files
def test_extract_encrypted_5(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("encrypted_5.7z").open(mode="rb"), password="secret")
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_extract_encrypted_6(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("encrypted_6.7z").open(mode="rb"), password="secret")
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_encrypt_file_0(tmp_path):
    filters = [{"id": py7zr.FILTER_LZMA}, {"id": py7zr.FILTER_CRYPTO_AES256_SHA256}]
    tmp_path.joinpath("src").mkdir()
    tmp_path.joinpath("tgt").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.set_encoded_header_mode(False)
    archive.writeall(".")
    archive.close()
    #
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt1"))
    reader.close()
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")


@pytest.mark.files
def test_encrypt_file_1(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w", password="secret")
    archive.writeall(os.path.join(testdata_path, "src"), "src")
    archive.set_encoded_header_mode(False)
    assert archive.header.main_streams.unpackinfo.folders[0].coders[0]["numinstreams"] == 1
    assert archive.header.main_streams.unpackinfo.folders[0].coders[0]["numoutstreams"] == 1
    archive._write_flush()
    assert len(archive.header.main_streams.unpackinfo.folders[0].coders[0]["properties"]) == 18
    assert archive.header.main_streams.substreamsinfo.unpacksizes == [11]
    assert archive.header.main_streams.unpackinfo.folders[0].unpacksizes == [17, 11]
    assert archive.header.main_streams.packinfo.packsizes == [32]
    archive._fpclose()
    #
    with py7zr.SevenZipFile(target, "r", password="secret") as arc:
        arc.extractall(path=tmp_path / "tgt")
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")


@pytest.mark.files
def test_encrypt_file_2(tmp_path):
    filters = [{"id": py7zr.FILTER_BZIP2}, {"id": py7zr.FILTER_CRYPTO_AES256_SHA256}]
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.writeall(".")
    archive.close()
    #
    tmp_path.joinpath("tgt").mkdir()
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt"))
    reader.close()
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")


@pytest.mark.files
def test_encrypt_file_3(tmp_path):
    filters = [
        {"id": py7zr.FILTER_DELTA},
        {"id": py7zr.FILTER_LZMA2},
        {"id": py7zr.FILTER_CRYPTO_AES256_SHA256},
    ]
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.writeall(".")
    archive.close()
    #
    tmp_path.joinpath("tgt").mkdir()
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt"))
    reader.close()
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")


@pytest.mark.files
def test_encrypt_file_4(tmp_path):
    filters = [
        {"id": py7zr.FILTER_X86},
        {"id": py7zr.FILTER_BZIP2},
        {"id": py7zr.FILTER_CRYPTO_AES256_SHA256},
    ]
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.writeall(".")
    archive.close()
    #
    tmp_path.joinpath("tgt").mkdir()
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt"))
    reader.close()
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")


@pytest.mark.files
def test_encrypt_file_5(tmp_path):
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path))
    with py7zr.SevenZipFile(target, mode="w", password="test123", header_encryption=True) as archive:
        archive.writeall("src", arcname="src")


@pytest.mark.files
def test_encrypt_file_6(tmp_path):
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path))
    with py7zr.SevenZipFile(target, mode="w", password="test123") as archive:
        archive.set_encrypted_header(True)
        archive.writeall("src", arcname="src")


@pytest.mark.files
def test_encrypt_file_7(tmp_path):
    filters = [{"id": py7zr.FILTER_ZSTD}, {"id": py7zr.FILTER_CRYPTO_AES256_SHA256}]
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.writeall(".")
    archive.close()
    #
    tmp_path.joinpath("tgt").mkdir()
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt"))
    reader.close()


@pytest.mark.files
def test_encrypt_emptyfile_1(tmp_path):
    tmp_path.joinpath("src").mkdir()
    with tmp_path.joinpath("src", "x").open(mode="wb") as f:
        f.write(b"")
    archive = py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", password="123")
    archive.set_encrypted_header(True)
    archive.write(tmp_path.joinpath("src", "x"), "y")
    archive.close()
    #
    with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "r", password="123") as arc:
        arc.extractall(path=tmp_path / "tgt")
    #
    assert tmp_path.joinpath("tgt", "y").is_file()


@pytest.mark.basic
def test_encrypt_simple_file_0(tmp_path):
    with tmp_path.joinpath("target.7z").open(mode="wb") as target:
        with py7zr.SevenZipFile(target, mode="w", password="secret") as archive:
            archive.writeall(os.path.join(testdata_path, "src"), "src")


@pytest.mark.files
def test_encrypt_file_8(tmp_path):
    filters = [
        {"id": py7zr.FILTER_X86},
        {"id": py7zr.FILTER_LZMA2},
        {"id": py7zr.FILTER_CRYPTO_AES256_SHA256},
    ]
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path.joinpath("target.7z")
    os.chdir(str(tmp_path.joinpath("src")))
    archive = py7zr.SevenZipFile(target, "w", password="secret", filters=filters)
    archive.writeall(".")
    archive.close()
    #
    tmp_path.joinpath("tgt").mkdir()
    reader = py7zr.SevenZipFile(target, "r", password="secret")
    reader.extractall(path=tmp_path.joinpath("tgt"))
    reader.close()
    #
    if shutil.which("7z"):
        result = subprocess.run(
            ["7z", "t", "-psecret", (tmp_path / "target.7z").as_posix()],
            stdout=subprocess.PIPE,
        )
        if result.returncode != 0:
            print(result.stdout)
            pytest.fail("7z command report error")



# Security protection test cases
import ctypes
import os
import sys

import pytest

from py7zr import SevenZipFile
from py7zr.exceptions import Bad7zFile
from py7zr.helpers import check_archive_path, get_sanitized_output_path
from py7zr.properties import FILTER_LZMA2, PRESET_DEFAULT

testdata_path = os.path.join(os.path.dirname(__file__), "data")


@pytest.mark.misc
def test_check_archive_path():
    bad_path = "../../.../../../../../../tmp/evil.sh"
    assert not check_archive_path(bad_path)


@pytest.mark.misc
def test_get_sanitized_output_path_1(tmp_path):
    bad_path = "../../.../../../../../../tmp/evil.sh"
    with pytest.raises(Bad7zFile):
        get_sanitized_output_path(bad_path, tmp_path)


@pytest.mark.misc
def test_get_sanitized_output_path_2(tmp_path):
    good_path = "good.sh"
    expected = tmp_path.joinpath(good_path)
    assert expected == get_sanitized_output_path(good_path, tmp_path)


@pytest.mark.misc
def test_extract_path_traversal_attack(tmp_path):
    my_filters = [
        {"id": FILTER_LZMA2, "preset": PRESET_DEFAULT},
    ]
    target = tmp_path.joinpath("target.7z")
    good_data = b"#!/bin/sh\necho good\n"
    good_path = "good.sh"
    bad_data = b"!#/bin/sh\necho bad\n"
    bad_path = "../../.../../../../../../tmp/evil.sh"
    with SevenZipFile(target, "w", filters=my_filters) as archive:
        archive.writestr(good_data, good_path)
        archive._writestr(bad_data, bad_path)  # bypass a path check
    with pytest.raises(Bad7zFile):
        with SevenZipFile(target, "r") as archive:
            archive.extractall(path=tmp_path)


@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
@pytest.mark.misc
def test_extract_symlink_attack(tmp_path):
    my_filters = [
        {"id": FILTER_LZMA2, "preset": PRESET_DEFAULT},
    ]
    source_dir = tmp_path / "src"
    symlink_file = source_dir / "symlink.sh"
    source_dir.mkdir(exist_ok=True)
    target_dir = tmp_path / "tgt"
    target = tmp_path / "target.7z"
    target_dir.mkdir(exist_ok=True)
    bad_data = b"!#/bin/sh\necho bad\n"
    bad_path = tmp_path.joinpath("evil.sh")
    with bad_path.open("wb") as evil:
        evil.write(bad_data)
    symlink_file.symlink_to(bad_path)
    with SevenZipFile(target, "w", filters=my_filters) as archive:
        archive.writeall(source_dir, "src")
    with pytest.raises(Bad7zFile):
        with SevenZipFile(target, "r") as archive:
            archive.extractall(path=target_dir)



import ctypes
import os
import pathlib
import sys

import pytest

import py7zr.helpers
import py7zr.win32compat

PATH_PREFIX = "\\\\?\\"


@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_symlink_readlink_absolute(tmp_path):
    origin = tmp_path / "parent" / "original.txt"
    origin.parent.mkdir(parents=True, exist_ok=True)
    with origin.open("w") as f:
        f.write("Original")
    slink = tmp_path / "target" / "link"
    slink.parent.mkdir(parents=True, exist_ok=True)
    target = origin.resolve()
    slink.symlink_to(target, False)
    if sys.platform.startswith("win"):
        assert py7zr.win32compat.readlink(str(tmp_path / "target" / "link")) == PATH_PREFIX + str(target)
        assert py7zr.helpers.readlink(str(slink)) == PATH_PREFIX + str(target)
        assert py7zr.helpers.readlink(slink) == pathlib.WindowsPath(PATH_PREFIX + str(target))
    else:
        assert py7zr.helpers.readlink(str(slink)) == str(target)
        assert py7zr.helpers.readlink(slink) == target
    assert slink.open("r").read() == "Original"


@pytest.mark.skipif(
    sys.platform.startswith("win") and (ctypes.windll.shell32.IsUserAnAdmin() == 0),
    reason="Administrator rights is required to make symlink on windows",
)
def test_symlink_readlink_relative(tmp_path):
    origin = tmp_path / "parent" / "original.txt"
    origin.parent.mkdir(parents=True, exist_ok=True)
    with origin.open("w") as f:
        f.write("Original")
    slink = tmp_path / "target" / "link"
    slink.parent.mkdir(parents=True, exist_ok=True)
    target = pathlib.Path("../parent/original.txt")
    slink.symlink_to(target, False)
    if sys.platform.startswith("win"):
        assert py7zr.win32compat.readlink(str(tmp_path / "target" / "link")) == str(target)
        assert py7zr.win32compat.readlink(tmp_path / "target" / "link") == target
    assert slink.open("r").read() == "Original"
    assert py7zr.helpers.readlink(slink) == target
    assert py7zr.helpers.readlink(str(slink)) == str(target)


def test_hardlink_readlink(tmp_path):
    target = tmp_path / "parent" / "original.txt"
    target.parent.mkdir(parents=True, exist_ok=True)
    with target.open("w") as f:
        f.write("Original")
    hard = tmp_path / "target" / "link"
    hard.parent.mkdir(parents=True, exist_ok=True)
    if sys.platform.startswith("win"):
        os.system("mklink /H %s %s" % (str(hard), str(target.resolve())))
    else:
        os.link(str(target.resolve()), str(hard))
    assert hard.open("r").read() == "Original"
    assert os.path.samefile(str(hard), str(target.resolve()))
    assert not py7zr.helpers.islink(hard)
    if sys.platform.startswith("win"):
        with pytest.raises(ValueError):
            py7zr.win32compat.readlink(str(hard))


@pytest.mark.skipif(not sys.platform.startswith("win"), reason="test on windows")
def test_junction_readlink(tmp_path):
    target = tmp_path / "parent"
    target.mkdir(parents=True, exist_ok=True)
    with target.joinpath("original.txt").open("w") as f:
        f.write("Original")
    junction = tmp_path / "target" / "link"
    junction.parent.mkdir(parents=True, exist_ok=True)
    os.system("mklink /J %s %s" % (str(junction), str(target.resolve())))
    assert not os.path.islink(str(junction))
    assert py7zr.win32compat.is_reparse_point(str(junction))
    assert py7zr.win32compat.readlink(str(junction)) == PATH_PREFIX + str(target.resolve())
    assert py7zr.helpers.readlink(str(junction)) == PATH_PREFIX + str(target.resolve())
    assert py7zr.win32compat.is_reparse_point(junction)
    assert py7zr.win32compat.readlink(junction) == pathlib.WindowsPath(PATH_PREFIX + str(target.resolve()))
    assert py7zr.helpers.readlink(junction) == pathlib.WindowsPath(PATH_PREFIX + str(target.resolve()))



import getpass
import lzma
import os
import shutil
import sys

import pytest

import py7zr
import py7zr.archiveinfo
import py7zr.callbacks
import py7zr.cli
import py7zr.properties

from . import check_output, libarchive_extract, ltime2, p7zip_test

testdata_path = os.path.join(os.path.dirname(__file__), "data")
os.umask(0o022)


@pytest.mark.cli
def test_cli_list_1(capsys):
    arc = os.path.join(testdata_path, "test_1.7z")
    expected = """total 4 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
"""
    expected += "{} D....            0            0  scripts\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A          111          441  scripts/py7zr\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A           58               setup.cfg\n".format(ltime2(2019, 3, 14, 0, 7, 13))
    expected += "{} ....A          559               setup.py\n".format(ltime2(2019, 3, 14, 0, 9, 1))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", arc])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_list_2(capsys):
    arc = os.path.join(testdata_path, "test_3.7z")
    expected = "total 28 files and directories in solid archive\n"
    expected += "   Date      Time    Attr         Size   Compressed  Name\n"
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    expected += "{} D....            0            0  5.9.7\n".format(ltime2(2018, 10, 18, 14, 52, 42))  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64\n".format(ltime2(2018, 10, 18, 14, 52, 43))  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/include\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/include/QtX11Extras\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/cmake\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/cmake/Qt5X11Extras\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/lib/pkgconfig\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/mkspecs\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} D....            0            0  5.9.7/gcc_64/mkspecs/modules\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A           26         8472  5.9.7/gcc_64/include/QtX11Extras/QX11Info\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A          176               5.9.7/gcc_64/include/QtX11Extras/QtX11Extras\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          201               5.9.7/gcc_64/include/QtX11Extras/QtX11ExtrasDepends\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A           32               5.9.7/gcc_64/include/QtX11Extras/QtX11ExtrasVersion\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          722               5.9.7/gcc_64/lib/libQt5X11Extras.la\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A         2280               5.9.7/gcc_64/include/QtX11Extras/qtx11extrasglobal.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A          222               5.9.7/gcc_64/include/QtX11Extras/qtx11extrasversion.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A         2890               5.9.7/gcc_64/include/QtX11Extras/qx11info_x11.h\n".format(
        ltime2(2018, 10, 16, 10, 26, 21)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so.5\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A        14568               5.9.7/gcc_64/lib/libQt5X11Extras.so.5.9.7\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A           24               5.9.7/gcc_64/lib/libQt5X11Extras.so.5.9\n".format(
        ltime2(2018, 10, 18, 14, 52, 42)
    )  # noqa: E501
    expected += "{} ....A         6704               5.9.7/gcc_64/lib/cmake/Qt5X11Extras/Qt5X11ExtrasConfig.cmake\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += (
        "{} ....A          287               5.9.7/gcc_64/lib/cmake/Qt5X11Extras/Qt5X11ExtrasConfigVersion.cmake\n".format(
            ltime2(2018, 10, 16, 10, 26, 24)
        )
    )  # noqa: E501
    expected += "{} ....A          283               5.9.7/gcc_64/lib/pkgconfig/Qt5X11Extras.pc\n".format(
        ltime2(2018, 10, 16, 10, 26, 27)
    )  # noqa: E501
    expected += "{} ....A          555               5.9.7/gcc_64/mkspecs/modules/qt_lib_x11extras.pri\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A          526               5.9.7/gcc_64/mkspecs/modules/qt_lib_x11extras_private.pri\n".format(
        ltime2(2018, 10, 16, 10, 26, 24)
    )  # noqa: E501
    expected += "{} ....A         1064               5.9.7/gcc_64/lib/libQt5X11Extras.prl\n".format(
        ltime2(2018, 10, 18, 10, 28, 16)
    )  # noqa: E501
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", arc])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_help(capsys):
    if sys.version_info >= (3, 10):
        expected = "usage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptions:"
    else:
        expected = "usage: py7zr [-h] [--version] {l,x,c,a,t," "i} ...\n\npy7zr\n\noptional arguments:\n  -h, --help"
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["-h"])
    out, err = capsys.readouterr()
    assert out.startswith(expected)


@pytest.mark.cli
def test_cli_no_subcommand(capsys):
    expected = py7zr.cli.Cli._get_version()
    if sys.version_info >= (3, 10):
        expected += "\nusage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptions:"
    else:
        expected += "\nusage: py7zr [-h] [--version] {l,x,c,a,t,i} ...\n\npy7zr\n\noptional arguments:\n  -h, --help"
    cli = py7zr.cli.Cli()
    cli.run([])
    out, err = capsys.readouterr()
    assert out.startswith(expected)


@pytest.mark.cli
def test_cli_list_verbose(capsys):
    arcfile = os.path.join(testdata_path, "test_1.7z")
    expected = """Listing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 657
Headers Size = 216
Method = LZMA2
Solid = +
Blocks = 1

total 4 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
""".format(
        arcfile, arcfile
    )
    expected += "{} D....            0            0  scripts\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A          111          441  scripts/py7zr\n".format(ltime2(2019, 3, 14, 0, 10, 8))
    expected += "{} ....A           58               setup.cfg\n".format(ltime2(2019, 3, 14, 0, 7, 13))
    expected += "{} ....A          559               setup.py\n".format(ltime2(2019, 3, 14, 0, 9, 1))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", "--verbose", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_test(capsys):
    arcfile = os.path.join(testdata_path, "test_2.7z")
    expected = """Testing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 1663
Headers Size = 203
Method = LZMA2
Solid = -
Blocks = 1

Everything is Ok
""".format(
        arcfile, arcfile
    )
    cli = py7zr.cli.Cli()
    cli.run(["t", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_info(capsys):
    expected = py7zr.cli.Cli._get_version()
    if lzma.is_check_supported(lzma.CHECK_CRC64):
        check0 = "\n0              CRC64"
    else:
        check0 = ""
    if lzma.is_check_supported(lzma.CHECK_SHA256):
        check1 = "\n0             SHA256"
    else:
        check1 = ""
    expected_checks = """0              CRC32{}{}""".format(check1, check0)
    expected += """\n
Formats:
7z    37 7a bc af 27 1c

Codecs and hashes:
00              COPY
21             LZMA2
03             DELTA
030101          LZMA
03030103         BCJ
03030205         PPC
03030401        IA64
03030501         ARM
03030701        ARMT
03030805       SPARC
040108       DEFLATE
040202         BZip2
04f71101   ZStandard
030401          PPMd
04f71102      Brotli
040109     DEFLATE64
06f10701       7zAES
{}
""".format(
        expected_checks
    )
    cli = py7zr.cli.Cli()
    cli.run(["i"])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_extract(tmp_path):
    arcfile = os.path.join(testdata_path, "test_1.7z")
    cli = py7zr.cli.Cli()
    cli.run(["x", arcfile, str(tmp_path.resolve())])
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        },
        {
            "filename": "setup.py",
            "mode": 33188,
            "mtime": 1552522141,
            "digest": "b916eed2a4ee4e48c51a2b51d07d450de0be4dbb83d20e67f6fd166ff7921e49",
        },
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        },
    ]
    check_output(expected, tmp_path)


@pytest.mark.cli
def test_cli_encrypted_extract(monkeypatch, tmp_path):
    def _getpasswd():
        return "secret"

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    expected = [
        {
            "filename": "test1.txt",
            "mode": 33188,
            "digest": "0f16b2f4c3a74b9257cd6229c0b7b91855b3260327ef0a42ecf59c44d065c5b2",
        },
        {
            "filename": "test/test2.txt",
            "mode": 33188,
            "digest": "1d0d28682fca74c5912ea7e3f6878ccfdb6e4e249b161994b7f2870e6649ef09",
        },
    ]
    check_output(expected, tmp_path)


@pytest.mark.cli
def test_cli_encrypted_wrong_password(monkeypatch, tmp_path, capsys):
    def _getpasswd():
        return "badsecret"

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    expected = "The archive is corrupted, or password is wrong. ABORT.\n"

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    result = cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert result == 1
    assert out == expected


@pytest.mark.cli
@pytest.mark.skipif(sys.hexversion == 0x030900F0, reason="bpo-42057")
def test_cli_encrypted_zero_length_password(monkeypatch, tmp_path, capsys):
    def _getpasswd():
        return ""

    monkeypatch.setattr(getpass, "getpass", _getpasswd)

    expected = "The archive is corrupted, or password is wrong. ABORT.\n"

    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    cli = py7zr.cli.Cli()
    result = cli.run(["x", "--password", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert result == 1
    assert out == expected


@pytest.mark.cli
def test_non7z_ext(capsys, tmp_path):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["x", arcfile, str(tmp_path.resolve())])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_non7z_test(capsys):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["t", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_non7z_list(capsys):
    expected = "not a 7z file\n"
    arcfile = os.path.join(testdata_path, "test_1.txt")
    cli = py7zr.cli.Cli()
    cli.run(["l", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_archive_creation(tmp_path, capsys):
    expected = ""
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    os.chdir(str(tmp_path))
    target = "target.7z"
    source = "src"
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source])
    out, err = capsys.readouterr()
    assert out == expected
    #
    p7zip_test(tmp_path / "target.7z")
    libarchive_extract(tmp_path / "target.7z", tmp_path.joinpath("tgt2"))


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_archive_already_exist(tmp_path, capsys):
    expected = "Archive file exists!\n"
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path / "target.7z"
    with target.open("w") as f:
        f.write("Already exist!")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["c", str(target), source])
    out, err = capsys.readouterr()
    assert err == expected


@pytest.mark.cli
def test_archive_append(tmp_path, capsys):
    expected = ""
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_2.7z"), path=tmp_path.joinpath("src"))
    target = tmp_path / "target.7z"
    shutil.copy(os.path.join(testdata_path, "test_1.7z"), target)
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["a", str(target), source])
    out, err = capsys.readouterr()
    assert err == expected
    #
    p7zip_test(tmp_path / "target.7z")
    libarchive_extract(tmp_path / "target.7z", tmp_path.joinpath("tgt2"))


@pytest.mark.cli
def test_archive_without_extension(tmp_path, capsys):
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "test_1.7z"), path=tmp_path.joinpath("src"))
    target = str(tmp_path / "target")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source])
    expected_target = tmp_path / "target.7z"
    assert expected_target.exists()


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_volume_creation(tmp_path, capsys):
    expected = ""
    tmp_path.joinpath("src").mkdir()
    py7zr.unpack_7zarchive(os.path.join(testdata_path, "lzma2bcj.7z"), path=tmp_path.joinpath("src"))
    target = str(tmp_path / "target.7z")
    source = str(tmp_path / "src")
    cli = py7zr.cli.Cli()
    cli.run(["c", target, source, "-v", "2m"])
    out, err = capsys.readouterr()
    assert out == expected
    assert err == expected


@pytest.mark.cli
@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_volume_creation_wrong_volume_unit(tmp_path, capsys):
    expected = "Error: Specified volume size is invalid.\n"
    target = str(tmp_path / "target.7z")
    source = tmp_path / "src"
    source.mkdir()
    cli = py7zr.cli.Cli()
    with pytest.raises(SystemExit):
        cli.run(["c", target, str(source), "-v", "2P"])
    out, err = capsys.readouterr()
    assert err == expected


@pytest.mark.cli
@pytest.mark.files
def test_list_multivolume(capsys):
    arcfile = os.path.join(testdata_path, "archive.7z.001")
    basefile = os.path.join(testdata_path, "archive.7z")
    expected = """Listing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 52337
Headers Size = 518
Method = LZMA2, BCJ
Solid = +
Blocks = 2

total 19 files and directories in solid archive
   Date      Time    Attr         Size   Compressed  Name
------------------- ----- ------------ ------------  ------------------------
""".format(
        basefile, basefile
    )
    expected += "{} D....            0            0  mingw64\n".format(ltime2(2017, 1, 23, 6, 2, 46))
    expected += "{} D....            0            0  mingw64/bin\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/include\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/lib\n".format(ltime2(2020, 6, 7, 2, 45, 18))
    expected += "{} D....            0            0  mingw64/share\n".format(ltime2(2020, 6, 7, 2, 45, 26))
    expected += "{} D....            0            0  mingw64/share/doc\n".format(ltime2(2017, 1, 23, 6, 2, 43))
    expected += "{} D....            0            0  mingw64/share/doc/szip\n".format(ltime2(2017, 1, 23, 6, 2, 43))
    expected += "{} ....A         2289        26895  mingw64/include/SZconfig.h\n".format(ltime2(2017, 1, 23, 6, 2, 34))
    expected += "{} ....A         3470               mingw64/include/ricehdf.h\n".format(ltime2(2004, 3, 16, 16, 14, 27))
    expected += "{} ....A         1774               mingw64/include/szip_adpt.h\n".format(ltime2(2010, 7, 2, 21, 31, 38))
    expected += "{} ....A         5282               mingw64/include/szlib.h\n".format(ltime2(2008, 11, 11, 16, 12, 56))
    expected += "{} ....A        60008               mingw64/lib/libszip.a\n".format(ltime2(2017, 1, 23, 6, 2, 47))
    expected += "{} ....A        10900               mingw64/lib/libszip.dll.a\n".format(ltime2(2017, 1, 23, 6, 2, 39))
    expected += "{} ....A         1986               mingw64/share/doc/szip/COPYING\n".format(ltime2(2008, 1, 24, 23, 8, 43))
    expected += "{} ....A         1544               mingw64/share/doc/szip/HISTORY.txt\n".format(
        ltime2(2010, 7, 14, 13, 43, 15)
    )
    expected += "{} ....A         3544               mingw64/share/doc/szip/INSTALL\n".format(
        ltime2(2008, 11, 11, 16, 12, 56)
    )
    expected += "{} ....A          564               mingw64/share/doc/szip/README\n".format(ltime2(2007, 8, 20, 18, 47, 21))
    expected += "{} ....A          513               mingw64/share/doc/szip/RELEASE.txt\n".format(
        ltime2(2010, 7, 14, 13, 43, 15)
    )
    expected += "{} ....A        66352        24924  mingw64/bin/libszip-0.dll\n".format(ltime2(2017, 1, 23, 6, 2, 47))
    expected += "------------------- ----- ------------ ------------  ------------------------\n"
    cli = py7zr.cli.Cli()
    cli.run(["l", "--verbose", arcfile])
    out, err = capsys.readouterr()
    assert out == expected


@pytest.mark.cli
def test_cli_encrypted_no_password(capsys):
    arcfile = os.path.join(testdata_path, "encrypted_1.7z")
    expected = """Testing archive: {}
--
Path = {}
Type = 7z
Phisical Size = 251
Headers Size = 203
Method = LZMA, 7zAES
Solid = +
Blocks = 1

The archive is encrypted but password is not given. FAILED.
""".format(
        arcfile, arcfile
    )
    cli = py7zr.cli.Cli()
    cli.run(["t", arcfile])
    out, err = capsys.readouterr()
    assert out == expected



import os

import pytest

import py7zr
from py7zr.exceptions import UnsupportedCompressionMethodError

testdata_path = os.path.join(os.path.dirname(__file__), "data")
os.umask(0o022)


@pytest.mark.files
def test_archiveinfo_deflate():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "deflate.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names[0] == "DEFLATE"


@pytest.mark.files
def test_archiveinfo_deflate64():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "deflate64.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names[0] == "DEFLATE64"


@pytest.mark.files
def test_archiveinfo_lzma_bcj2():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma_bcj2_1.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA", "BCJ2*"]


@pytest.mark.files
def test_archiveinfo_lzma_bcj():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma_bcj_x86.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA", "BCJ"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2bcj.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "BCJ"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj_arm():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2_bcj_arm.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "ARM"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj_armt():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2_bcj_armt.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "ARMT"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj_ia64():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2_bcj_ia64.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "IA64"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj_ppc():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2_bcj_ppc.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "PPC"]


@pytest.mark.files
def test_archiveinfo_lzma2_bcj_sparc():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2_bcj_sparc.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA2", "SPARC"]


@pytest.mark.files
def test_archiveinfo_7zaes_lzma():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "encrypted_1.7z"), "r") as ar:
        ai = ar.archiveinfo()
        assert ai.method_names == ["LZMA", "7zAES"]


@pytest.mark.files
def test_archivetest_deflate():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "deflate.7z"), "r") as ar:
        assert ar.testzip() is None


@pytest.mark.files
def test_archivetest_deflate64():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "deflate64.7z"), "r") as ar:
        assert ar.testzip() is None


@pytest.mark.files
def test_archivetest_lzma_bcj2():
    with pytest.raises(UnsupportedCompressionMethodError):
        with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma_bcj2_1.7z"), "r") as ar:
            assert ar.testzip() is None


@pytest.mark.files
def test_archivetest_lzma_bcj():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma_bcj_x86.7z"), "r") as ar:
        assert ar.testzip() is None


@pytest.mark.files
def test_archivetest_lzma2_bcj():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "lzma2bcj.7z"), "r") as ar:
        assert ar.testzip() is None


@pytest.mark.files
def test_archivetest_7zaes():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "encrypted_1.7z"), "r", password="secret") as ar:
        assert ar.testzip() is None


@pytest.mark.files
def test_list_filename_encryption(tmp_path):
    with py7zr.SevenZipFile(os.path.join(testdata_path, "filename_encryption.7z"), "r", password="hello") as ar:
        file_list = ar.list()
        assert file_list[0].filename == "New Text Document.TXT"



import os
import pathlib
import re
import shutil

import pytest

import py7zr
import py7zr.archiveinfo
import py7zr.callbacks
import py7zr.cli
import py7zr.properties

from . import check_output, decode_all

testdata_path = os.path.join(os.path.dirname(__file__), "data")
os.umask(0o022)


@pytest.mark.basic
def test_basic_initinfo():
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    assert archive is not None


@pytest.mark.api
def test_basic_not_implemented_yet1(tmp_path):
    with pytest.raises(NotImplementedError):
        py7zr.SevenZipFile(tmp_path.joinpath("test_x.7z"), mode="x")


@pytest.mark.api
def test_basic_append_mode(tmp_path):
    target = tmp_path.joinpath("test_a.7z")
    shutil.copy(os.path.join(testdata_path, "test_1.7z"), target)
    with py7zr.SevenZipFile(target, mode="a") as archive:
        archive.write(os.path.join(testdata_path, "test1.txt"), "test1.txt")


@pytest.mark.api
def test_basic_wrong_option_value(tmp_path):
    with pytest.raises(ValueError):
        py7zr.SevenZipFile(tmp_path.joinpath("test_p.7z"), mode="p")


@pytest.mark.basic
def test_basic_extract_1(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        },
        {
            "filename": "setup.py",
            "mode": 33188,
            "mtime": 1552522141,
            "digest": "b916eed2a4ee4e48c51a2b51d07d450de0be4dbb83d20e67f6fd166ff7921e49",
        },
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        },
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.basic
def test_basic_extract_2(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_2.7z"), "rb"))
    expected = [
        {
            "filename": "qt.qt5.597.gcc_64/installscript.qs",
            "digest": "39445276e79ea43c0fa8b393b35dc621fcb2045cb82238ddf2b838a4fbf8a587",
        }
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.basic
def test_basic_decode_3(tmp_path):
    """Test when passing path string instead of file-like object."""
    archive = py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"))
    expected = [
        {
            "filename": "setup.cfg",
            "mode": 33188,
            "mtime": 1552522033,
            "digest": "ff77878e070c4ba52732b0c847b5a055a7c454731939c3217db4a7fb4a1e7240",
        }
    ]
    decode_all(archive, expected, tmp_path)


@pytest.mark.api
def test_py7zr_is_7zfile():
    assert py7zr.is_7zfile(os.path.join(testdata_path, "test_1.7z"))


@pytest.mark.api
def test_py7zr_is_7zfile_fileish():
    assert py7zr.is_7zfile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))


@pytest.mark.api
def test_py7zr_is_7zfile_path():
    assert py7zr.is_7zfile(pathlib.Path(testdata_path).joinpath("test_1.7z"))


@pytest.mark.basic
def test_py7zr_is_not_7zfile(tmp_path):
    target = tmp_path.joinpath("test_not.7z")
    with target.open("wb") as f:
        f.write(b"12345dahodjg98adfjfak;")
    with target.open("rb") as f:
        assert not py7zr.is_7zfile(f)


@pytest.mark.basic
def test_digests():
    arcfile = os.path.join(testdata_path, "test_2.7z")
    archive = py7zr.SevenZipFile(arcfile)
    assert archive.test() is None
    assert archive.testzip() is None


@pytest.mark.basic
def test_digests_corrupted():
    arcfile = os.path.join(testdata_path, "crc_corrupted.7z")
    with py7zr.SevenZipFile(arcfile) as archive:
        assert archive.test() is None
        archive.reset()
        assert archive.testzip() == "src/scripts/py7zr"


@pytest.mark.unit
def test_py7zr_write_mode(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.write(os.path.join(testdata_path, "test1.txt"), "test1.txt")
    assert archive.files is not None
    assert len(archive.files) == 1
    for f in archive.files:
        assert f.filename in ("test1.txt")
        assert not f.emptystream


@pytest.mark.api
def test_py7zr_writeall_single(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.writeall(os.path.join(testdata_path, "test1.txt"), "test1.txt")
    assert archive.files is not None
    assert len(archive.files) == 1
    for f in archive.files:
        assert f.filename in ("test1.txt")
        assert not f.emptystream


@pytest.mark.api
def test_py7zr_writeall_dir(tmp_path):
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w")
    archive.writeall(os.path.join(testdata_path, "src"), "src")
    assert archive.files is not None
    assert len(archive.files) == 2
    for f in archive.files:
        assert f.filename in ("src", "src/bra.txt")
    archive._fpclose()


@pytest.mark.api
def test_py7zr_extract_specified_file(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    expected = [
        {
            "filename": "scripts/py7zr",
            "mode": 33261,
            "mtime": 1552522208,
            "digest": "b0385e71d6a07eb692f5fb9798e9d33aaf87be7dfff936fd2473eab2a593d4fd",
        }
    ]
    archive.extract(path=tmp_path, targets=["scripts", "scripts/py7zr"])
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert not tmp_path.joinpath("setup.cfg").exists()
    assert not tmp_path.joinpath("setup.py").exists()
    check_output(expected, tmp_path)


@pytest.mark.api
def test_py7zr_extract_and_getnames(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    allfiles = archive.getnames()
    filter_pattern = re.compile(r"scripts.*")
    targets = [f for f in allfiles if filter_pattern.match(f)]
    archive.extract(path=tmp_path, targets=targets)
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert not tmp_path.joinpath("setup.cfg").exists()
    assert not tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_py7zr_extract_and_reset_iteration(tmp_path):
    archive = py7zr.SevenZipFile(open(os.path.join(testdata_path, "test_1.7z"), "rb"))
    iterations = archive.getnames()
    for target in iterations:
        archive.extract(path=tmp_path, targets=[target])
        archive.reset()
    archive.close()
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert tmp_path.joinpath("setup.cfg").exists()
    assert tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_context_manager_1(tmp_path):
    with py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"), "r") as z:
        z.extractall(path=tmp_path)
    assert tmp_path.joinpath("scripts").is_dir()
    assert tmp_path.joinpath("scripts/py7zr").exists()
    assert tmp_path.joinpath("setup.cfg").exists()
    assert tmp_path.joinpath("setup.py").exists()


@pytest.mark.api
def test_context_manager_2(tmp_path):
    target = tmp_path.joinpath("target.7z")
    with py7zr.SevenZipFile(target, "w") as z:
        z.writeall(os.path.join(testdata_path, "src"), "src")


@pytest.mark.api
def test_py7zr_list_values():
    with py7zr.SevenZipFile(os.path.join(testdata_path, "test_1.7z"), "r") as z:
        file_list = z.list()
    assert file_list[0].filename == "scripts"
    assert file_list[1].filename == "scripts/py7zr"
    assert file_list[2].filename == "setup.cfg"
    assert file_list[3].filename == "setup.py"
    assert file_list[0].uncompressed == 0
    assert file_list[1].uncompressed == 111
    assert file_list[2].uncompressed == 58
    assert file_list[3].uncompressed == 559
    assert file_list[0].is_directory is True
    assert file_list[1].archivable is True
    assert file_list[2].archivable is True
    assert file_list[3].archivable is True
    assert file_list[0].compressed == 0
    assert file_list[1].compressed == 441
    assert file_list[2].compressed is None
    assert file_list[3].compressed is None
    assert file_list[0].crc32 is None
    assert file_list[1].crc32 == 0xB36AAEDB
    assert file_list[2].crc32 == 0xDCBF8D07
    assert file_list[3].crc32 == 0x80FC72BE



import io
import os
import pathlib
import zlib

import pytest
from Cryptodome.Cipher import AES

import py7zr
import py7zr.compressor
from py7zr import UnsupportedCompressionMethodError
from py7zr.properties import FILTER_DEFLATE64
from tests import p7zip_test

testdata_path = pathlib.Path(os.path.dirname(__file__)).joinpath("data")
os.umask(0o022)


@pytest.mark.unit
def test_aes_cipher():
    key = b"e\x11\xf1Pz<*\x98*\xe6\xde\xf4\xf6X\x18\xedl\xf2Be\x1a\xca\x19\xd1\\\xeb\xc6\xa6z\xe2\x89\x1d"
    iv = b"|&\xae\x94do\x8a4\x00\x00\x00\x00\x00\x00\x00\x00"
    indata = (
        b"T\x9f^\xb5\xbf\xdc\x08/\xfe<\xe6i'\x84A^\x83\xdc\xdd5\xe9\xd5\xd0b\xa9\x7fH$\x11\x82\x8d"
        b"\xce[\x85\xe7\xf2}\xe3oJ*\xc0:\xf4\xfd\x82\xe8I"
    )
    expected = (
        b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
        b"\xf7\\m\xe0t\xa6$yF_-\xa0\x0b8f "
    )
    cipher = AES.new(key, AES.MODE_CBC, iv)
    result = cipher.decrypt(indata)
    assert expected == result


@pytest.mark.unit
def test_deflate_compressor():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data
    compressor = py7zr.compressor.DeflateCompressor()
    outdata = compressor.compress(plain_data)
    outdata += compressor.flush()
    expected = b"c\xd0\x92\xe2TO\x91\xdc`Q|\xaa[Xa\xbd\x95t\xef\xf4\x1f5\xca\xbe/\x1f\x86_y2\xdd\xc9\xe9\x12\x03\x01y\x00"
    assert outdata == expected


@pytest.mark.unit
def test_deflate_compressor_flushed():
    with pytest.raises(zlib.error):
        plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
        plain_data += plain_data
        compressor = py7zr.compressor.DeflateCompressor()
        outdata = compressor.compress(plain_data)
        outdata += compressor.flush()
        outdata += compressor.flush()


@pytest.mark.unit
def test_deflate_decompressor():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data
    source_data = b"c\xd0\x92\xe2TO\x91\xdc`Q|\xaa[Xa\xbd\x95t\xef\xf4\x1f5\xca\xbe/\x1f\x86_y2\xdd\xc9\xe9\x12\x03\x01y\x00"
    decompressor = py7zr.compressor.DeflateDecompressor()
    outdata = decompressor.decompress(source_data)
    assert outdata == plain_data


@pytest.mark.unit
def test_copy_compressor():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    compressor = py7zr.compressor.CopyCompressor()
    outdata = compressor.compress(plain_data)
    outdata += compressor.flush()
    assert outdata == plain_data


@pytest.mark.unit
def test_copy_decompressor():
    source_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    decompressor = py7zr.compressor.CopyDecompressor()
    outdata = decompressor.decompress(source_data)
    assert outdata == source_data


@pytest.mark.unit
def test_pyzstd_decompressor_1():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data
    compressed = (
        b"\x28\xb5\x2f\xfd"
        b"\x00XE\x01\x00\x04\x02\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4"
        b"\xe4\x97BB\xd2\x01\x00\x18\xb8z\x02"
    )
    property = b"\x01\x04\x04\x00\x00"
    decompressor = py7zr.compressor.ZstdDecompressor(property, None)
    outdata = decompressor.decompress(compressed)
    assert outdata == plain_data


@pytest.mark.unit
def test_pyzstd_decompressor_2():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data
    compressed_0 = b"\x28\xb5\x2f\xfd"
    compressed_1 = b" @E\x01\x00\x04\x02\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4"
    compressed_2 = b"\xe4\x97BB\xd2\x01\x00\x18\xb8z\x02"
    property = b"\x01\x04\x04\x00\x00"
    decompressor = py7zr.compressor.ZstdDecompressor(property, None)
    outdata = decompressor.decompress(compressed_0)
    outdata += decompressor.decompress(compressed_1)
    outdata += decompressor.decompress(compressed_2)
    assert outdata == plain_data


@pytest.mark.unit
def test_sevenzipcompressor_aes_lzma2():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data + plain_data
    filters = [
        {"id": py7zr.FILTER_LZMA2, "preset": py7zr.PRESET_DEFAULT},
        {"id": py7zr.FILTER_CRYPTO_AES256_SHA256},
    ]
    indata = io.BytesIO(plain_data)
    outdata = io.BytesIO()
    compressor = py7zr.compressor.SevenZipCompressor(filters=filters, password="secret")
    insize, outsize, crc = compressor.compress(indata, outdata)
    outsize += compressor.flush(outdata)
    outdata.seek(0, 0)
    coders = compressor.coders
    unpacksizes = compressor.unpacksizes
    decompressor = py7zr.compressor.SevenZipDecompressor(
        coders=coders,
        packsize=outsize,
        unpacksizes=unpacksizes,
        crc=None,
        password="secret",
    )
    revert_data = decompressor.decompress(outdata)
    assert revert_data == plain_data


@pytest.mark.unit
def test_aes_compressor():
    compressor = py7zr.compressor.AESCompressor("secret")
    assert compressor.method == py7zr.properties.CompressionMethod.CRYPT_AES256_SHA256
    assert len(compressor.encode_filter_properties()) == 2 + 16


@pytest.mark.unit
def test_sevenzipcompressor_aes_only():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
    plain_data += plain_data
    filters = [{"id": py7zr.FILTER_CRYPTO_AES256_SHA256}]
    compressor = py7zr.compressor.SevenZipCompressor(filters=filters, password="secret")
    indata = io.BytesIO(plain_data)
    outdata = io.BytesIO()
    _, _, _ = compressor.compress(indata, outdata)
    _ = compressor.flush(outdata)
    result = outdata.getvalue()
    assert len(result) == 64
    assert result != plain_data


@pytest.mark.unit
def test_aes_encrypt_data():
    plain_data = b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB"
    plain_data += plain_data + plain_data
    compressor = py7zr.compressor.AESCompressor("secret")
    outdata = compressor.compress(plain_data)
    outdata += compressor.flush()
    assert len(outdata) == 96  # 96 = 16 * 6 = len(plain_data) + 3


@pytest.mark.unit
def test_aes_decrypt(monkeypatch):
    properties = b"S\x07|&\xae\x94do\x8a4"
    indata = (
        b"T\x9f^\xb5\xbf\xdc\x08/\xfe<\xe6i'\x84A^\x83\xdc\xdd5\xe9\xd5\xd0b\xa9\x7fH$\x11\x82\x8d"
        b"\xce[\x85\xe7\xf2}\xe3oJ*\xc0:\xf4\xfd\x82\xe8I"
    )
    expected = (
        b"\x00*\x1a\t'd\x19\xb08s\xca\x8b\x13 \xaf:\x1b\x8d\x97\xf8|#M\xe9\xe1W\xd4\xe4\x97BB\xd2"
        b"\xf7\\m\xe0t\xa6$yF_-\xa0\x0b8f "
    )
    decompressor = py7zr.compressor.AESDecompressor(properties, "secret")
    assert decompressor.decompress(indata) == expected


@pytest.mark.files
def test_extract_bzip2(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("bzip2_2.7z").open(mode="rb"))
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_extract_ppmd(tmp_path):
    archive = py7zr.SevenZipFile(testdata_path.joinpath("ppmd.7z").open(mode="rb"))
    archive.extractall(path=tmp_path)
    archive.close()


@pytest.mark.files
def test_extract_ppmd2(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("testdata-x5-ppmd.7z").open(mode="rb")) as archive:
        archive.extractall(path=tmp_path)


@pytest.mark.files
def test_extract_deflate(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("deflate.7z").open(mode="rb")) as archive:
        archive.extractall(path=tmp_path)


@pytest.mark.basic
def test_compress_ppmd(tmp_path):
    my_filters = [{"id": py7zr.FILTER_PPMD, "order": 6, "mem": 24}]  # mem in 2^size
    target = tmp_path.joinpath("target.7z")
    archive = py7zr.SevenZipFile(target, "w", filters=my_filters)
    archive.writeall(os.path.join(testdata_path, "src"), "src")
    archive.close()
    #
    p7zip_test(tmp_path / "target.7z")


@pytest.mark.basic
def test_compress_ppmd_2(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("bzip2_2.7z").open(mode="rb")) as arc:
        arc.extractall(path=tmp_path)
    my_filters = [{"id": py7zr.FILTER_PPMD, "order": 6, "mem": "4m"}]
    target = tmp_path.joinpath("target.7z")
    with py7zr.SevenZipFile(target, "w", filters=my_filters) as archive:
        archive.encoded_header_mode = False
        archive.writeall(tmp_path.joinpath("10000SalesRecords.csv"))
    #
    p7zip_test(tmp_path / "target.7z")


@pytest.mark.basic
def test_compress_decompress_brotli(tmp_path):
    my_filters = [{"id": py7zr.FILTER_BROTLI, "level": 11}]
    #
    with py7zr.SevenZipFile(testdata_path.joinpath("bzip2_2.7z").open(mode="rb")) as arc:
        arc.extractall(path=tmp_path)
    target = tmp_path.joinpath("target.7z")
    with py7zr.SevenZipFile(target, "w", filters=my_filters) as archive:
        archive.write(tmp_path.joinpath("10000SalesRecords.csv"), "10000SalesRecords.csv")
    tmp_path.joinpath("tgt").mkdir(exist_ok=True)
    with py7zr.SevenZipFile(target, "r") as archive:
        archive.extractall(path=tmp_path.joinpath("tgt"))


@pytest.mark.basic
def test_decompress_brotli(tmp_path):
    with pytest.raises(UnsupportedCompressionMethodError):
        with py7zr.SevenZipFile(testdata_path.joinpath("zstdmt-brotli.7z").open(mode="rb")) as archive:
            archive.extractall(path=tmp_path.joinpath("tgt"))


@pytest.mark.basic
@pytest.mark.skip(reason="Deflate64 compression is not implemented yet.")
def test_compress_decompress_deflate64(tmp_path):
    # prepare source data
    with py7zr.SevenZipFile(testdata_path.joinpath("bzip2_2.7z").open(mode="rb")) as arc:
        arc.extractall(path=tmp_path)
    # compress with deflate64
    target = tmp_path.joinpath("target.7z")
    my_filters = [{"id": FILTER_DEFLATE64}]
    with py7zr.SevenZipFile(target, "w", filters=my_filters) as archive:
        archive.write(tmp_path.joinpath("10000SalesRecords.csv"), "10000SalesRecords.csv")
    # check extract
    tmp_path.joinpath("tgt").mkdir(exist_ok=True)
    with py7zr.SevenZipFile(target, "r") as archive:
        archive.extractall(path=tmp_path.joinpath("tgt"))
    # check with p7zip
    p7zip_test(tmp_path / "target.7z")


@pytest.mark.basic
def test_compress_deflate64(tmp_path):
    my_filters = [{"id": FILTER_DEFLATE64}]
    with pytest.raises(UnsupportedCompressionMethodError):
        with py7zr.SevenZipFile(tmp_path.joinpath("target.7z"), "w", filters=my_filters) as archive:
            archive.write(testdata_path.joinpath("src"), "src")


@pytest.mark.basic
def test_decompress_deflate64(tmp_path):
    with py7zr.SevenZipFile(testdata_path.joinpath("deflate64.7z").open(mode="rb")) as archive:
        archive.extractall(path=tmp_path.joinpath("tgt"))
